#include "shell_ext.h"
#include <signal.h>
#include <fcntl.h>
#ifndef mips
#include <sys/filio.h>
#endif

#ifndef PLAINTERM

#ifdef IRIS
#define TERMIO
#endif

/*
** structures for saving tty parameters
*/
#ifdef TERMIO 
#include <termio.h>
struct termio old_tty,new_tty;
#else
#include <sgtty.h>
struct sgttyb old_tty,new_tty;
#endif
static char area[1024];

/*
** terminal capabilities
*/
char *tgetstr();
#endif PLAINTER

static char 	*ce;		/* clear eol */
static char     *so ;		/* begin standout mode */
static char     *se ;		/* end standout mode */
static char     *cm ;		/* cursor motion */
static int      li ;		/* number of lines */
static int      co ;		/* number of columns */
static char     *cs ;		/* change scrolling region */
static char     *rs ;		/* reset to sane mode */
static char     *sc ;		/* save cursor position */
static char     *rc ;		/* restore cursor position */

char 	PC;
char 	*BC;
char 	*UP;


static int	status_active;
static int	status_offset = 2;

cleareol()
{
    if(ce != NULL){
	printf("%s",ce);
    }
    fflush(stdout);
}

StatusActive()
{
    return(status_active);
}

SaveCursorPosition()
{
    if(sc){
	printf("%s",sc);
	fflush(stdout);
    }
}

RestoreCursorPosition()
{
    if(rc){
	printf("%s",rc);
	fflush(stdout);
    }
}

SaneTerm()
{
    if(rs){
	printf("%s",rs);
	fflush(stdout);
    }
}

TermParse(cs,parameter)
char *cs;
int *parameter;
{
char *ptr;
int current_parm = 0;
unsigned char ch;

    for(ptr=cs;*ptr != '\0';ptr++){
	if(*ptr == '\0') break;
	switch(*ptr){
	case '%':
	    ptr++;
	    switch(*ptr){
	    case 'd':
		printf("%d",parameter[current_parm++]);
		break;
	    case '2':
		printf("%2d",parameter[current_parm++]);
		break;
	    case '3':
		printf("%3d",parameter[current_parm++]);
		break;
	    case 'i':
		parameter[current_parm]++;
		break;
	    case '+':
		ptr++;
		ch = (unsigned char)(parameter[current_parm++]) + *ptr;
		printf("%c",ch);
		break;
	    case '.':
		ch = parameter[current_parm++];
		printf("%c",ch);
		break;
	    case '%':
		printf("%c",'%');
		break;
	    default:
		printf("%c",*ptr);
		break;
	    }
	    break;
	default:
	    printf("%c",*ptr);
	    break;
	}
    }
    fflush(stdout);
}

TermGoto(line,col)
int line;
int col;
{
int parameter[2];

    if(cm){
	parameter[0] = line;
	parameter[1] = col;
	TermParse(cm,parameter);
	return(1);
    } else {
	return(0);
    }
}

int StatusMessage(s,col,line)
char *s;
int col;
int line;
{
int inverse;

    if(status_active){
	if(line == 0) {
	    inverse = 1;
	} else {
	    inverse = 0;
	}
	SaveCursorPosition();    
	TermGoto(li - status_offset + line,col);
	if(inverse)
	    Standout(TRUE);
	printf("%s",s);
	if(inverse)
	    Standout(FALSE);
	RestoreCursorPosition();    
	fflush(stdout);
	return(1);
    } else {
	return(0);
    }
}

StatusLine(line,inverse)
int line;
int inverse;
{
int i;

    if(status_active){
	SaveCursorPosition(); 
	if(TermGoto(li - status_offset +line,1)){
	    if(inverse)
		Standout(TRUE);
	    for(i=0;i<co;i++){
		printf(" ");
	    }
	    if(inverse)
		Standout(FALSE);
	}
	RestoreCursorPosition();
    }
}

int ScrollRegion(bottom,top)
int bottom;
int top;
{
int parameter[2];

    if(cs){
	parameter[0] = bottom;
	parameter[1] = top;
	/*
	** parse the escape sequence
	*/
	TermParse(cs,parameter);
	return(1);
    } else {
	return(0);
    }
}

Standout(val)
int val;
{
    switch(val){
    case 0:			/* standout off */
	if(se){
	    printf("%s",se);
	}
	break;
    case 1:			/* standout on */
	if(so){
	    printf("%s",so);
	}
	break;
    }
}

#ifdef PLAINTERM

/*
** for vanilla terminal handling just define empty functions for
*/
genesis_tty(){}
restore_tty(){}
normal_tty(){}
terminal_setup(){}
void tset(){}

#else PLAINTERM
/*
** give back the original terminal state
*/
normal_tty(){
    /*
    ** ignore io interrupts
    */
    signal(SIGIO,SIG_DFL);
    /*
    ** reset the terminal parameters to their original state
    */
#ifdef TERMIO
    ioctl(fileno(stdin),TCSETA,&old_tty);
#else TERMIO
    ioctl(fileno(stdin),TIOCSETP,&old_tty);
#endif TERMIO
}

restore_tty()
{
    normal_tty();
    DisableStatusLine();
    printf("\n");		/* scroll an additional line */
}

genesis_tty()
{
    tset();
}


/*
** prepare the terminal settings used in thee simulator command line io
*/
terminal_setup()
{
    /* 
    ** save the old terminal parameters
    */
#ifdef TERMIO
    ioctl(fileno(stdin),TCGETA,&old_tty);
#else
    ioctl(fileno(stdin),TIOCGETP,&old_tty);
#endif
    /*
    ** set the new terminal parameters
    */
    tset();
}
#endif PLAINTERM

EnableScrollRegion()
{
    /*
    ** in order to do the status line, the terminal must be capable
    ** of saving and restoring cursor locations as well as region scrolling
    */
    if(sc && rc && ScrollRegion(0,li-status_offset)) {
	status_active = 1;
    } else {
	status_active = 0;
    }
}

DisableStatusLine()
{
    ScrollRegion(0,li);
    StatusLine(0,0);
    status_active = 0;
    TermGoto(li-1,0);		/* go to the bottom of the screen */
}

EnableStatusLine()
{
    printf("\n\n\n");
    EnableScrollRegion();
    StatusLine(0,1);
    TermGoto(li - status_offset - 1,0);
}

#ifndef PLAINTERM
char *termstr(s,area)
char *s;
char **area;
{
char *value;

    value = tgetstr(s,area);	/* get the terminal capability string */
    if(value){
	/*
	** skip the speed component
	*/
	while((*value <= '9' && *value >='0') || *value == '*') value++;
    }
    return(value);
}

/*
** terminal setup must be called prior to doing the first tset
** or bad things will happen
*/
void tset()
{
char *term;
char *getenv();
char bp[1024];
char *areaptr;
int nc;
int i;
char buffer[1000];
extern int iofunc();

    /*
    ** get any leftover characters from the previous terminal io
    ** state
    */
    ioctl(fileno(stdin),FIONREAD,&nc);
    if(nc) {
	for(i=0;i<nc;i++){
	    buffer[i] = getc(stdin);
	}
    }
    /*
    ** load the termcap entry
    */
    term = getenv("TERM");
    if(term == NULL) {
	printf("undefined terminal type. Using vt100.\n");
	term = "vt100";
    }
    if(tgetent(bp,term) == -1){
	printf("could not find termcap entry for %s\n",term);
    }
    /*
    ** get some term capabilities
    ** like backspace and eol
    */
    areaptr = area;

    ce = termstr("ce",&areaptr);		/* clear eol */
    so = termstr("so",&areaptr);		/* begin standout mode */
    se = termstr("se",&areaptr);		/* end standout mode */
    cm = termstr("cm",&areaptr);		/* cursor motion */
    co = tgetnum("co");				/* number of columns */
    li = tgetnum("li");				/* number of lines */
    cs = termstr("cs",&areaptr);		/* change scrolling region */
    rs = termstr("rs",&areaptr);		/* reset to sane mode */
    UP = termstr("UP",&areaptr);		/* move cursor up n lines */
    sc = termstr("sc",&areaptr);		/* save cursor position */
    rc = termstr("rc",&areaptr);		/* restore cursor position */

    /*
    ** copy the old terminal parameters and
    bcopy(&old_tty,&new_tty,sizeof(struct sgttyb));
    ** prepare to set the new terminal parameters
    */
    new_tty = old_tty; 
#ifdef TERMIO
    /*
    ** enable single character input
    */
    new_tty.c_lflag &= ~(ECHO | ICANON);
    new_tty.c_cc[VEOF] =  1;
    new_tty.c_cc[VEOL] = 1; 
    /*
    ** set the new parameters
    */
    ioctl(fileno(stdin),TCSETA,&new_tty);
#ifdef SIGIO
    /* 
    ** enable the sending of SIGIO when input becomes available 
    */
    fcntl(fileno(stdin),F_SETFL,FASYNC);
    /*
    ** put back any leftover characters from the previous terminal io
    ** state
    */
    signal(SIGIO,SIG_DFL);
    /*
    ** enable io interrupts
    */
    signal(SIGIO,iofunc);
#endif SIGIO
    if(nc) {
	for(i=0;i<nc;i++){
	    ioctl(fileno(stdin),TIOCSTI,buffer+i);
	}
    }
#else TERMIO
    /*
    ** enable single character input
    */
    new_tty.sg_flags |= CBREAK;
    /*
    ** dont echo characters as they are typed
    */
    new_tty.sg_flags &= ~ECHO;
    /*
    ** set the new parameters
    */
    ioctl(fileno(stdin),TIOCSETP,&new_tty);
#ifdef SIGIO
    /* 
    ** enable the sending of SIGIO when input becomes available 
    */
    fcntl(fileno(stdin),F_SETFL,FASYNC);
    /*
    ** put back any leftover characters from the previous terminal io
    ** state
    */
    signal(SIGIO,SIG_DFL);
    /*
    ** enable io interrupts
    */
    signal(SIGIO,iofunc);
#endif SIGIO
    if(nc) {
	for(i=0;i<nc;i++){
	    ioctl(fileno(stdin),TIOCSTI,buffer+i);
	}
    }
#endif TERMIO
    /* 
    ** attach the interrupt handlers to the various signals 
    */
    /*
    signal(SIGQUIT,quitfunc);
    signal(SIGINT,intfunc);
    */
}

#endif PLAINTERM

int do_status_message(argc,argv)
int argc;
char **argv;
{
    if(argc < 4){
	printf("usage: %s string col line\n",argv[0]);
	return(0);
    }
    return(StatusMessage(argv[1],atoi(argv[2]),atoi(argv[3])));
}
